/*#######################################################################################
Futaba S-BUS Decode for Mikrokopter version 0.01a 20100828
Futaba S-BUS Decode for Mikrokopter version 0.02a 20150104 Update for higher frequency of S-BUS data

Original Specktrum.c from FlightCtrl V0.80g modified by kri@kri.dk to decode 
Futaba S-BUS. 

Use at your own risk.

Thanks to sebseb7 for the work on decoding the protocol.

Since libfc is closed source and changes to that library would be need to integrate this
nicely with the old Spectrum code, I have chosen for now just to make a replacement  
witch breaks the current Spectrum code, in KopterTool it will also still be listed as 
Spektrum.

Since the S-BUS receiver sends the signal out inverted, I have used a CMOS 4049 DIP just 
shrink wrapped on the wires between the FlightCtrl and the Receiver.

CMOS i have used is this one:
http://www.electrokit.se/ic-c-mos-4xxx-4049ub-dip-14-hex-inverting-buffer_40360096

Look at this:
http://gallery.mikrokopter.de/main.php/v/tech/FC_ME_with_Spektrum.jpg.html

The red wire should not go to the gyro, but to the +5V pad next to the black GND.

Wires then go:

FC        	CD4049UB    	S-BUS
GND Black <->   Vss Pin8  <->   GND Black
+5V Red   <->   Vdd Pin1  <->   +5V Red
RX1 Grey  <->   H   Pin4   
                B   Pin5  <->   Signal Orange 

#######################################################################################*/

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Copyright (c) Holger Buss, Ingo Busker
// + Nur für den privaten Gebrauch / NON-COMMERCIAL USE ONLY
// + www.MikroKopter.com
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Es gilt für das gesamte Projekt (Hardware, Software, Binärfiles, Sourcecode und Dokumentation),
// + dass eine Nutzung (auch auszugsweise) nur für den privaten und nichtkommerziellen Gebrauch zulässig ist.
// + Sollten direkte oder indirekte kommerzielle Absichten verfolgt werden, ist mit uns (info@mikrokopter.de) Kontakt
// + bzgl. der Nutzungsbedingungen aufzunehmen.
// + Eine kommerzielle Nutzung ist z.B.Verkauf von MikroKoptern, Bestückung und Verkauf von Platinen oder Bausätzen,
// + Verkauf von Luftbildaufnahmen, usw.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Werden Teile des Quellcodes (mit oder ohne Modifikation) weiterverwendet oder veröffentlicht,
// + unterliegen sie auch diesen Nutzungsbedingungen und diese Nutzungsbedingungen incl. Copyright müssen dann beiliegen
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Sollte die Software (auch auszugesweise) oder sonstige Informationen des MikroKopter-Projekts
// + auf anderen Webseiten oder sonstigen Medien veröffentlicht werden, muss unsere Webseite "http://www.mikrokopter.de"
// + eindeutig als Ursprung verlinkt und genannt werden
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Keine Gewähr auf Fehlerfreiheit, Vollständigkeit oder Funktion
// + Benutzung auf eigene Gefahr
// + Wir übernehmen keinerlei Haftung für direkte oder indirekte Personen- oder Sachschäden
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Die Portierung oder Nutzung der Software (oder Teile davon) auf andere Systeme (ausser der Hardware von www.mikrokopter.de) ist nur
// + mit unserer Zustimmung zulässig
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Die Funktion printf_P() unterliegt ihrer eigenen Lizenz und ist hiervon nicht betroffen
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// + Redistributions of source code (with or without modifications) must retain the above copyright notice,
// + this list of conditions and the following disclaimer.
// +   * Neither the name of the copyright holders nor the names of contributors may be used to endorse or promote products derived
// +     from this software without specific prior written permission.
// +   * The use of this project (hardware, software, binary files, sources and documentation) is only permittet
// +     for non-profit use (directly or indirectly)
// +     Commercial use (for excample: selling of MikroKopters, selling of PCBs, assembly, ...) is only permitted
// +     with our written permission
// +   * If sources or documentations are redistributet, our webpage (http://www.MikroKopter.de) must be
// +     clearly linked and named as origin
// +   * porting the sources to other systems or using the software on other systems (except hardware from www.mikrokopter.de) is not allowed
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// +  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// +  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// +  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// +  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// +  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// +  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// +  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// +  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// +  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// +  POSSIBILITY OF SUCH DAMAGE.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//


#include "Spektrum.h"
#include "main.h"
unsigned char SpektrumTimer = 0;

#ifdef RECEIVER_SPEKTRUM_EXP
unsigned char s_excnt = 0;	// Counter for Spektrum-Expander
unsigned char s_exparity = 0;	// Parity Bit for Spektrum-Expander
signed char s_exdata[11];	// Data for Spektrum-Expander
#endif				/*  */
//--------------------------------------------------------------//
//--------------------------------------------------------------//
void SpektrumBinding(void)
{
}

//############################################################################
// USART1 initialisation from killagreg
void SpektrumUartInit(void)
//############################################################################
{

	// -- Start of USART1 initialisation for Spekturm seriell-mode
	// USART1 Control and Status Register A, B, C and baud rate register
	uint8_t sreg = SREG;
	uint16_t ubrr = (uint16_t) ((uint32_t) SYSCLK / (8 * 100000) - 1);

	// disable all interrupts before reconfiguration
	cli();

	// disable RX-Interrupt
	UCSR1B &= ~(1 << RXCIE1);

	// disable TX-Interrupt
	UCSR1B &= ~(1 << TXCIE1);

	// disable DRE-Interrupt
	UCSR1B &= ~(1 << UDRIE1);

	// set direction of RXD1 and TXD1 pins
	// set RXD1 (PD2) as an input pin
	PORTD |= (1 << PORTD2);
	DDRD &= ~(1 << DDD2);

	// set TXD1 (PD3) as an output pin
	PORTD |= (1 << PORTD3);
	DDRD |= (1 << DDD3);

	// USART0 Baud Rate Register
	// set clock divider
	UBRR1H = (uint8_t) (ubrr >> 8);
	UBRR1L = (uint8_t) ubrr;

	// enable double speed operation
	UCSR1A |= (1 << U2X1);

	// enable receiver and transmitter
	//UCSR1B = (1<<RXEN1)|(1<<TXEN1);
	UCSR1B = (1 << RXEN1);

	// set asynchronous mode
	UCSR1C &= ~(1 << UMSEL11);
	UCSR1C &= ~(1 << UMSEL10);

	// even parity
	UCSR1C |= (1 << UPM11);
	UCSR1C &= ~(1 << UPM10);

	// 2 stop bit
	UCSR1C |= (1 << USBS1);

	// 8-bit
	UCSR1B &= ~(1 << UCSZ12);
	UCSR1C |= (1 << UCSZ11);
	UCSR1C |= (1 << UCSZ10);

	// flush receive buffer explicit
	while (UCSR1A & (1 << RXC1))
		UDR1;

	// enable RX-interrupts at the end
	UCSR1B |= (1 << RXCIE1);

	// -- End of USART1 initialisation
	// restore global interrupt flags
	SREG = sreg;
	return;
}

#define MIN_FRAMEGAP 30
#define MAX_BYTEGAP  3		// 412us

//############################################################################
// Wird im UART-Interrupt aufgerufen
//############################################################################
void SpektrumParser(unsigned char c)
{
	static unsigned char FrameCnt = 0, ReSync = 1, ch_bit = 0, ch = 1;
	static int tmp_signal=0;
	signed int tmp, signal=0;
	int bCheckDelay, index =0, bit ;
	if (ReSync == 1) {	// wait for beginning of new frame
		ReSync = 0;
		SpektrumTimer = MIN_FRAMEGAP;
		FrameCnt = 0;
		ch_bit = 0;
		ch = 1;
		DebugOut.Analog[16]++;
	} else {
		if (!SpektrumTimer)
			bCheckDelay = 1;
		else
			bCheckDelay = 0;
		if (FrameCnt == 0) {
			if (bCheckDelay) {
				FrameCnt++;
				SpektrumTimer = MAX_BYTEGAP;
			} else {
				SpektrumTimer = MIN_FRAMEGAP;
				FrameCnt = 0;
				ch_bit = 0;
				ch = 1;
			}



		} else if ((FrameCnt <= 22) && !bCheckDelay) { // Servo Data
			SpektrumTimer = MAX_BYTEGAP;
			FrameCnt++;
			for (bit = 0; bit < 8; bit++) {
				if (c & (1 << bit)) {
					tmp_signal |= (1 << ch_bit);
				}
                  		ch_bit++;
                  		if (ch_bit == 11) {
                      			ch_bit = 0;
					signal=tmp_signal;
					tmp_signal=0;
					index=ch;
                      			ch++;
                    		}
			}
		} else if ((FrameCnt = 23) && !bCheckDelay) {  //Flags
			SpektrumTimer = MAX_BYTEGAP;
			FrameCnt++;
			FrameCnt = 0;
			ch_bit = 0;
			ch = 1;
		} else {
			ReSync = 1;
			FrameCnt = 0;
			SpektrumTimer = MIN_FRAMEGAP;
		}
		if (index >0 && index < 13) {
			signal -= 0x3ff;
			signal /= 6;
			if (abs(signal - PPM_in[index]) < 6) {
				if (SenderOkay < 200)
					SenderOkay += 10;
				else {
					SenderOkay = 200;
					TIMSK1 &= ~_BV(ICIE1);	// disable PPM-Input
				}
			}
			tmp = (3 * (PPM_in[index]) + signal) / 4;
			if (tmp > signal + 1)
				tmp--;
			else if (tmp < signal - 1)
				tmp++;
			if (SenderOkay >= 180)
				PPM_diff[index] =
				    ((tmp - PPM_in[index]) / 3) * 3;
			else
				PPM_diff[index] = 0;
			PPM_in[index] = tmp;
			if(index >= 4)  NewPpmData = 0;  // Null bedeutet: Neue Daten

		}
		index = 0;

	}
}
